@@ -3,6 +3,7 @@ Tests for main algorithm
 '''
 
 from pathlib import Path
+import shutil
 
 from coconet.coconet import main
 from coconet.parser import parse_args
@@ -12,8 +13,10 @@ TEST_DIR = "{}/tests/sim_data".format(Path(__file__).parent.parent)
 
 PARAMS = {
     'fasta': Path("{}/assembly.fasta".format(TEST_DIR)),
-    'coverage': list(Path(TEST_DIR).glob("sample_*.bam")),
+    'bam': list(Path(TEST_DIR).glob("sample_*.bam")),
     'output': Path('./output_test'),
+    'min_ctg_len': 2048,
+    'fragment_length': 1024,
     'n_train': 64,
     'n_test': 8,
     'batch_size': 2,
@@ -25,7 +28,7 @@ PARAMS = {
     'cover_neurons': [8, 4],
     'cover_kernel': 2,
     'wsize': 2,
-    'wstep': 2
+    'wstep': 2,
 }
 
 def test_parser():
@@ -35,7 +38,7 @@ def test_parser():
 
     args = parse_args()
 
-    assert isinstance(args.kmer, int)
+    assert hasattr(args, 'action')
 
 def test_all():
     '''
@@ -43,19 +46,16 @@ def test_all():
     '''
 
     fail = True
-
     try:
-        main(**PARAMS)
+        main(loglvl='DEBUG', **PARAMS)
         fail = False
     except Exception as error:
         print(error)
 
-    for filepath in PARAMS['output'].glob('./*'):
-        filepath.unlink()
-    PARAMS['output'].rmdir()
+    shutil.rmtree(PARAMS['output'])
 
     assert not fail
 
 if __name__ == '__main__':
-    # test_parser()
+
     test_all()
